# -*- coding: utf-8 -*-
import hashlib
import copy


class SignMixin(object):
    secret_key = "TQAFrtRS4WrUn1hHKByszao3ozWvIMkF"

    @classmethod
    def join_dict(cls, payload: list) -> str:
        return "&".join(payload)

    @classmethod
    def sorted_dict(cls, payload: dict) -> list:
        return sorted(["{}={}".format(key, value) for key, value in payload.items()])

    def sign(self, data: dict = None) -> str:
        payload = copy.deepcopy(data)
        if data is None:
            payload = dict()
        payload.update({"secret": self.secret_key})
        pre_sign = self.join_dict(self.sorted_dict(payload))
        return hashlib.sha256(pre_sign.encode("utf8")).hexdigest()



# -*- coding: utf-8 -*-
from coinceres.api import APIClient


class ExchangeAPIClient(APIClient):
    exchange = None

    def contract_info(self, contract: str = None):
        return super(ExchangeAPIClient, self).contract_info(self.exchange, contract)

    def account(self):
        return super(ExchangeAPIClient, self).account(self.exchange)

    def order_info(self, system_oid: str = None, status: int = None, contract: str = None):
        return super(ExchangeAPIClient, self).order_info()




# -*- coding: utf-8 -*-
from functools import wraps


def error_helper(f):
    @wraps(f)
    def _wrapper(*args, **kwargs):
        r = f(*args, **kwargs)
        data = r.json()
        if r.status_code != 200:
            raise ValueError(data.get('message'))
        if data.get('code') != '200':
            raise ValueError(data.get('message'))
        return data.get('data')
    return _wrapper



# -*- coding: utf-8 -*-
from coinceres.api import APIClient
from coinceres.sign import SignMixin


def test_sign():
    payload = {
        "exchange": "BITMEX"
    }
    assert SignMixin().sign(payload) == "9173fddc96e445b07a3d11295b52ca45385f96d9a52307da58134a2fd7e239be"


def test_contract():
    r = APIClient(api_key="dWbkgDeLIzLavnYs", secret_key="dePW2XslyzFYnTuc41yRhqHIUWEVco4W", host="192.168.50.172").contract_info("BITMEX")
    print(r)


def test_account():
    r = APIClient(api_key="dWbkgDeLIzLavnYs", secret_key="dePW2XslyzFYnTuc41yRhqHIUWEVco4W", host="192.168.50.172").account()
    print(r)


def test_order():
    r = APIClient(api_key="dWbkgDeLIzLavnYs", secret_key="dePW2XslyzFYnTuc41yRhqHIUWEVco4W",
                   host="192.168.50.172").limit_order("BITMEX", "BTC_USDT", "10", "buy", "open", "10", "0.000011", "0.00000997")
    print(r)



# -*- coding: utf-8 -*-
from coinceres.exceptions import error_helper
from coinceres.http import HttpRequest
from coinceres.sign import SignMixin


class APIClient(HttpRequest, SignMixin):
    """Http客户端"""

    def __init__(self, api_key, secret_key, host='open.coinceres.com', url='api/v1'):
        """
        :param api_key:
        :param secret_key:
        """
        self.api_key = api_key
        self.secret_key = secret_key
        self.url = self.join_url("http:/", host, url)

    @error_helper
    def contract_info(self, exchange: str, contract: str = None):
        """
        獲取交易所內各幣對基本信息
        :param exchange: 交易所名稱 OKEX,BINANCE,HUOBI,BITFINEX,BITMEX
        :param contract: 幣對或合約名稱
        :return:
        """
        data = {
            "exchange": exchange
        }
        if contract:
            data.update(contract=contract)
        return self._do_get(api="basic/contracts", data=data)

    @error_helper
    def account(self, exchange: str = None):
        """
        獲取帳號信息
        :param exchange: 交易所名稱 OKEX,BINANCE,HUOBI,BITFINEX,BITMEX
        :return:
        """
        data = None
        if exchange:
            data = {
                "exchange": exchange
            }
        return self._do_get(api="trade/account", data=data)

    @error_helper
    def order_info(self, system_oid: str = None, status: int = None, exchange: str = None, contract: str = None):
        """
        查詢訂單詳情
        :param system_oid: 系统生成订单号，逗号分隔，最多可查询15个
        :param status: 订单状态
        :param exchange: 交易所名稱
        :param contract: 幣對或合約名稱
        :return:
        """
        data = dict()
        if system_oid:
            data.update(system_oid=system_oid)
        if status:
            data.update(status=status)
        if exchange:
            data.update(exchange=exchange)
        if contract:
            data.update(contract=contract)
        if not data:
            data = None
        return self._do_get("trade/order", data=data)

    @error_helper
    def _order(self, exchange: str, contract: str, entrust_vol: str, entrust_bs: str, future_dir: str,
               lever: str, price_type: str, entrust_price: str = None, profit_value: str = None,
               stop_value: str = None, client_oid: str = None):
        payload = {
            "exchange": exchange,
            "contract": contract,
            "price_type": price_type,
            "entrust_vol": entrust_vol,
            "entrust_bs": entrust_bs,
            "future_dir": future_dir,
            "lever": lever,
        }
        if entrust_price:
            payload.update(entrust_price=entrust_price)
        if profit_value:
            payload.update(profit_value=profit_value)
        if stop_value:
            payload.update(stop_value=stop_value)
        if client_oid:
            payload.update(client_oid=client_oid)
        return self._do_post("trade/input", data=payload)

    def market_order(self, exchange: str, contract: str, entrust_vol: str, entrust_bs: str, future_dir: str,
                     lever: str, entrust_price: str = None, profit_value: str = None, stop_value: str = None,
                     client_oid: str = None):
        """
        創建市價訂單
        :param exchange: 交易所名稱 OKEX,BINANCE,HUOBI,BITFINEX,BITMEX
        :param contract: 幣對或合約名稱
        :param entrust_vol: 交易量
        :param entrust_bs: 交易方向 "buy"/"sell"
        :param future_dir: "open"/"close"
        :param lever: 槓桿倍數
        :param entrust_price: 委託價格
        :param profit_value: 止盈价,合约必传
        :param stop_value: 止损价，合约必传
        :param client_oid: 来源标记
        :return:
        """
        return self._order(exchange, contract, entrust_vol, entrust_bs, future_dir, lever, "market",
                           entrust_price, profit_value, stop_value, client_oid)

    def limit_order(self, exchange: str, contract: str, entrust_vol: str, entrust_bs: str, future_dir: str,
                    lever: str, entrust_price: str, profit_value: str = None, stop_value: str = None,
                    client_oid: str = None):
        """
        創建限價訂單
        :param exchange: 交易所名稱 OKEX,BINANCE,HUOBI,BITFINEX,BITMEX
        :param contract: 幣對或合約名稱
        :param entrust_vol: 交易量
        :param entrust_bs: 交易方向 "buy"/"sell"
        :param future_dir: "open"/"close"
        :param lever: 槓桿倍數
        :param entrust_price: 委託價格
        :param profit_value: 止盈价,合约必传
        :param stop_value: 止损价，合约必传
        :param client_oid: 来源标记
        :return:
        """
        return self._order(exchange, contract, entrust_vol, entrust_bs, future_dir, lever, "limit",
                           entrust_price, profit_value, stop_value, client_oid)

    @error_helper
    def delete_order(self, system_oid: str):
        """
        取消訂單
        :param system_oid: 訂單號
        :return:
        """
        payload = {
            "system_oid": system_oid
        }
        return self._do_delete("trade/order/", data=payload)

    @error_helper
    def open_contract(self, exchange: str = None, contract: str = None, position_dir: str = None):
        """
        查询合约持仓信息
        :param exchange: 交易所名称
        :param contract: 合约名称
        :param position_dir: 持仓方向，多: "buy"/空: "sell"
        :return:
        """
        data = dict()
        if exchange:
            data.update(exchange=exchange)
        if contract:
            data.update(contract=contract)
        if position_dir:
            data.update(position_dir=position_dir)
        if not data:
            data = None
        return self._do_get("trade/position", data=data)

    @error_helper
    def transaction(self, exchange: str, contract: str, count: int):
        """
        查询成交纪录
        :param exchange: 交易所名称
        :param contract: 币币交易对或合约名称
        :param count: 查询数量 最大50
        :return:
        """
        data = dict(
            exchange=exchange,
            contract=contract,
            count=count
        )
        return self._do_get("trade/trans", data=data)



# -*- coding: utf-8 -*-
import requests


class HttpRequest(object):

    @classmethod
    def join_url(cls, *paths):
        return "/".join("{}".format(value) for value in paths)

    def _do_post(self, api: str, data: dict = None):
        headers = {'api_key': self.api_key, 'sign': self.sign(data)}
        return requests.post(self.join_url(self.url, api), json=data, headers=headers)

    def _do_get(self, api: str, data: dict = None):
        headers = {'api_key': self.api_key, 'sign': self.sign(data)}
        if data is None:
            return requests.get(self.join_url(self.url, api), headers=headers)
        return requests.get(self.join_url(self.url, api), params=data, headers=headers)

    def _do_delete(self, api: str, data: dict = None):
        headers = {'api_key': self.api_key, 'sign': self.sign(data)}
        return requests.delete(self.join_url(self.url, api), params=data, headers=headers)



# -*- coding: utf-8 -*-
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

REQUIRED_PYTHON = (3, 5)

setuptools.setup(
    name="coinceres",
    version="0.0.1",
    author="Peter Zhao",
    author_email="zhao6217@outlook.com",
    description="coinceres API SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/PetersonZhao/python-sdk",
    packages=setuptools.find_packages(),
    python_requires='>={}.{}'.format(*REQUIRED_PYTHON),
    classifiers=[
        "Programming Language :: Python :: 3",
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'requests',
    ]
)
